from argparse import Action, ArgumentParser

known_drivers = ['local', 's3']


class DriverAction(Action):
    def __call__(self, parser, namespace, values, option_string=None):
        driver, destination = values
        namespace.driver = driver.lower()
        if driver.lower() not in known_drivers:
            parser.error(
                "Unknown driver. Available drivers are 'local' and 's3'")
        namespace.destination = destination


def create_parser():
    parser = ArgumentParser()
    parser.add_argument('url', help='URL of the PostgreSQL database to backup')
    parser.add_argument(
        '--driver', '-d',
        help='how & where to store the backup',
        nargs=2,
        action=DriverAction,
        metavar=('driver', 'destination'),
        required=True
    )
    return parser


def main():
    import time
    import boto3
    from pgbackup import pgdump, storage

    args = create_parser().parse_args()
    dump = pgdump.dump(args.url)

    if args.driver == 's3':
        client = boto3.client('s3')
        timestamp = time.strftime("%Y-%m-%dT%H:%M:%S", time.localtime())
        file_name = pgdump.dump_file_name(args.url, timestamp)
        print(
            f'Backing database up to {args.destination} in s3 as {file_name}'
        )
        storage.s3(client, dump.stdout, args.destination, 'example.sql')

    else:
        outfile = open(args.destination, 'wb')
        print(f"Backing database up locally to {outfile.name}")
        storage.local(dump.stdout, outfile)



import subprocess
import sys


def dump(url):
    try:
        return subprocess.Popen(['pg_dump', url], stdout=subprocess.PIPE)
    except OSError as err:
        print(f'Error: {err}')
        sys.exit(1)


def dump_file_name(url, timestamp=None):
    db_name = url.split('/')[-1]
    db_name = db_name.split("?")[0]
    if timestamp:
        return f'{db_name}-{timestamp}.sql'
    else:
        return f'{db_name}.sql'



from importlib.metadata import entry_points
from setuptools import setup, find_packages

with open('README.rst', encoding='UTF-8') as f:
    readme = f.read()


setup(
    name='pgbackup',
    version='1.0.1',
    description='Database backups locally or to AWS S3.',
    long_description=readme,
    author='Kaushal',
    author_email='kaushalsharma880@gmail.com',
    url="https://github.com/iamkaushal/pg_backup",
    instal_requires=['boto3'],
    packages=find_packages('src'),
    package_dir={'': 'src'},


    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],


    entry_points={
        'console_scripts': [
            'pgbackup=pgbackup.cli:main',
        ]
    }
)
